@@ -3,14 +3,14 @@
 
 #include "MaemoApplication.h"
 #include "MaemoComment.h"
-
+#include <QObject>
 
 class MaemoApplicationDetails : public MaemoApplication {
     Q_OBJECT
 
     public:
         MaemoApplicationDetails();
-
+        ~MaemoApplicationDetails();
         //set parameters
         void setDesc (QString);
         void setLicence (QString);
