@@ -5,15 +5,22 @@
 #include <QFSFileEngine>
 #include <QDesktopServices>
 #include <QIODevice>
+#include <QDebug>
+
 
 // file installer
-#ifdef Q_WS_HILDON
+#ifdef Q_WS_MAEMO_5
+
     #include <hildon-mime.h>
     #include <iostream>
     #include <stdlib.h>
 #endif
 // file installer end
 
+const int MaemoApplicationHandler::LoggedIn = 1;
+const int MaemoApplicationHandler::LoggedInConnectionError = 2;
+const int MaemoApplicationHandler::LoggedInCredentialsError = 3;
+
 /***  
   MaemoApplicationHandler offers the communication interface for the maemo.org
   REST API. At the moment it is done in a way, that it requests every time the data
@@ -24,17 +31,18 @@
          - error handling
   ***/
 
-MaemoApplicationHandler::MaemoApplicationHandler () {
-    connector = new HttpConnector();
-    imageQueue = new QList<QueueElement*>;
+MaemoApplicationHandler::MaemoApplicationHandler (QString proxy,int port, int timeout) {
+    connector = new HttpConnector(timeout);
+    requestQueue = new QList<QueueElement*>;
     imageInUse = new QueueElement();
     queueConnected = false;
+    OCS_URL="http://maemo.org/ocs/v1/content/";
+    PRODUCT_URL="http://maemo.org/downloads/api/product/get/";
+    //COMMENT_URL="https://maemo.org/downloads/product/Maemo5/";
     mode = "testing";
-    #ifdef Q_WS_HILDON
-        //do nothing
-    #else
-        connector->setProxy("172.16.42.133",8080);
-    #endif
+    if (!proxy.isEmpty() && port!=0) {
+        connector->setProxy(proxy,port);
+    }
 }
 
 
@@ -51,12 +59,34 @@ void MaemoApplicationHandler::requestApplications(   QString OS,
                                                      int pageSize,
                                                      QString keyword,
                                                      QString sortmode) {
-    QTextStream out (stdout);
-    connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseAppListResponse(bool,QString*)));
+    qDebug() << "request Application: " + OS + "Category: " + cat;
     QString query = queryBuilder (OS,cat,page,pageSize,keyword,sortmode);
-    connector->connectToHost ("http://maemo.org/ocs/v1/content/data"+query);
+    QString url = OCS_URL +"data/" + query;
+    qDebug()<<"URL requested: " + url;
+    /***
+      check if connector can handle request, otherwise add to queue
+      high priority, set at begin of queue
+      **/
+    if (connector->isBusy()) {
+        qDebug () << "connector busy - append to queue as top element";
+        if (!queueConnected) {
+            queueConnected = true;
+            connect (connector, SIGNAL(isFree()),this,SLOT (proceedQueue()));
+        }
+        requestQueue->insert(requestQueue->begin(),new QueueElement (url,"","",QueueElement::LIST));
+    } else {
+        qDebug () << "connector free - send request";
+        connector->getRequest (url);
+        connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseAppListResponse(bool,QString*)));
+    }
 }
 
+
+void MaemoApplicationHandler::requestApplications(RestRequest *req) {
+    this->requestApplications(req->os(),req->category(),req->page(),req->pageSize(),req->keword(),req->sorting());
+}
+
+
 /***
   requests the application data of a single application
 
@@ -64,9 +94,48 @@ void MaemoApplicationHandler::requestApplications(   QString OS,
   - signal for a successfull request: detailsReady
   ***/
 void MaemoApplicationHandler::requestApplicationData(QString uid) {
-    connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseSingleAppResponse(bool,QString*)));
-    QString url = "http://maemo.org/downloads/api/product/get/"+uid+"/";
-    connector->connectToHost(url);
+    QString url = PRODUCT_URL+uid+"/";
+    qDebug ()<<"applicationDetails: uid:"<< uid.data() << "url: "<<url.data();
+    /***
+      request send if connector is free, otherwise send to queue (priority)
+      **/
+    if (connector->isBusy()) {
+        qDebug () << "connector busy - add request to queue";
+        if (!queueConnected) {
+            queueConnected = true;
+            connect (connector, SIGNAL(isFree()),this,SLOT (proceedQueue()));
+        }
+        requestQueue->insert (requestQueue->begin(), new QueueElement (url,"","",QueueElement::DETAILS));
+    } else {
+        connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseSingleAppResponse(bool,QString*)));
+        connector->getRequest(url);
+    }
+
+}
+
+/***
+  request categories
+  ***/
+void MaemoApplicationHandler::requestCategories(QString OS) {
+   QString url = OCS_URL + "categories?parent=" + OS;
+   /***
+     check if connector can handle request, otherwise add to queue
+     high priority, set at begin of queue
+     **/
+   qDebug () << "Categories requested: " << url;
+   if (connector->isBusy()) {
+       qDebug () << "connector busy - append to queue as top element";
+       if (!queueConnected) {
+           queueConnected = true;
+           connect (connector, SIGNAL (isFree()),this,SLOT(proceedQueue()));
+       }
+       requestQueue->insert(requestQueue->begin(),new QueueElement (url,"","",QueueElement::LIST));
+   }
+   else {
+       qDebug () << "free, send request";
+       connector->getRequest (url);
+       connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseCategoryResponse(bool,QString*)));
+   }
 }
 
 /***
@@ -82,27 +151,105 @@ void MaemoApplicationHandler::requestImage (QString url, QString id, QString inf
         imageInUse->setID(id);
         imageInUse->setInfo(info);
         connect (connector, SIGNAL(getRawData(bool,QByteArray)),this,SLOT(proceedImageResponse(bool,QByteArray)));
-        connector->connectToHost(url);
+        connector->getRequest(url);
     } else {
         if (!queueConnected) {
             queueConnected = true;
             connect (connector, SIGNAL(isFree()),this,SLOT (proceedQueue()));
         }
         if (addToQueue) {
-            imageQueue->append(new QueueElement (url,id,info));
+            /***
+              set requested images from the details-view to higher priority
+              ***/
+            if (info=="DETAILS"){
+                requestQueue->insert(requestQueue->begin(),new QueueElement (url,id,info,QueueElement::IMAGE));
+            } else {
+                requestQueue->append(new QueueElement (url,id,info,QueueElement::IMAGE));
+            }
         }
     }
 }
 
+/**
+  Sends a request to login the user onto maemo.org
+    @param: QString uname: username
+    @param: QString pw: password
 
+    emits signal loggedin (QString, bool)
+**/
+
+void MaemoApplicationHandler::requestLogin(QString uname, QString pw) {
+    qDebug() << "request login: uname: " << uname;
+    username = uname;
+    QByteArray postParams;
+    postParams.append("midcom_services_auth_frontend_form_submit=Login&");
+    postParams.append("username="+uname+"&");
+    postParams.append("password="+pw);
+
+    if (!connector->isBusy()) {
+        connector->postRequest(QUrl ("https://maemo.org/"),postParams);
+        connect (connector,SIGNAL(getHttpResponseCode(int)),this,SLOT(loginResponse(int)));
+    }
+    else {
+        emit loggedIn (LoggedInConnectionError);
+    }
+}
+
+/**
+  Returns true if a coockie is saved, false if not
+**/
+bool MaemoApplicationHandler::isUserLoggedIn() {
+    return connector->hasCookieForUrl(QUrl ("https://maemo.org"));
+}
+
+/**
+  Return username if the user is logged in
+  **/
+QString MaemoApplicationHandler::userName () {
+    if (connector->hasCookieForUrl(QUrl ("https://maemo.org"))&&!username.isEmpty())
+        return username;
+    else
+        return QString();
+
+}
+
+
+/**
+  deletes the cookies of the connection to log the user out
+ **/
+void MaemoApplicationHandler::logOut() {
+    connector->resetCookies();
+}
+
+
+void MaemoApplicationHandler::emptyQueue () {
+    qDebug ("request to empty the queue");
+    requestQueue->clear();
+    queueConnected = false;
+    disconnect (connector, SIGNAL(isFree()),this,SLOT (proceedQueue()));
+    qDebug ("done");
+}
+
+/***
+  slot, which handles the queue, called, when connector has handled a request
+  **/
 void MaemoApplicationHandler::proceedQueue () {
-     if (imageQueue->count()>0) {
-         QueueElement *q = imageQueue->takeFirst();
-         requestImage (q->getURL(),q->getID(),q->getInfo(),false);
-         delete q;
+    //qDebug ()<<"proceed with next queueElement; Rest in queue: " << QString::number(requestQueue->count());
+     if (requestQueue->count()>0) {
+        QueueElement *q = requestQueue->takeFirst();
+        if (q->getType()==QueueElement::IMAGE) {
+            requestImage (q->getURL(),q->getID(),q->getInfo(),false);
+        } else if (q->getType()==QueueElement::LIST) {
+            connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseAppListResponse(bool,QString*)));
+            connector->getRequest(q->getURL());
+        } else if (q->getType()==QueueElement::DETAILS) {
+            connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseSingleAppResponse(bool,QString*)));
+            connector->getRequest(q->getURL());
+        }
+        delete q;
      } else {
-         queueConnected = false;
-         disconnect (connector, SIGNAL(isFree()),this,SLOT (proceedQueue()));
+        queueConnected = false;
+        disconnect (connector, SIGNAL(isFree()),this,SLOT (proceedQueue()));
      }
 
  }
@@ -116,7 +263,7 @@ void MaemoApplicationHandler::proceedQueue () {
   ***/
 
 void MaemoApplicationHandler::requestInstallFile (QString url) {
-    connector->connectToHost (url);
+    connector->getRequest (url);
     QTextStream out (stdout);
     out <<"*********"+url << endl;
     connect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(proceedInstallFileResponse(bool,QString*)));
@@ -144,23 +291,19 @@ void MaemoApplicationHandler::proceedImageResponse (bool err,QByteArray img)  {
   @todo: everything, just a bit more than a placeholder up to now
 ***/
 void MaemoApplicationHandler::proceedInstallFileResponse (bool error, QString *content) {
-    QTextStream out (stdout);
-    out << *content << endl;
     disconnect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(proceedInstallFileResponse(bool,QString*)));
-    #ifdef Q_WS_HILDON
+    #ifdef Q_WS_MAEMO_5
         QFile file ("/opt/appdownloader/meta/application.install");
     #else
         QFile file ("~/.maemoclient/test.install");
     #endif
-        //QTextStream out (stdout);
         if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-            out << "fileerror" << endl;
-            //todo
+            qDebug() << "fileerror";
         }
         QTextStream file_writer (&file);
         file_writer << *content << endl;
         file.close();
-    #ifdef Q_WS_HILDON
+    #ifdef Q_WS_MAEMO_5
         const int suc = 1;
         int result = 0;
         DBusError err;
@@ -168,25 +311,80 @@ void MaemoApplicationHandler::proceedInstallFileResponse (bool error, QString *c
         DBusConnection *connection = dbus_bus_get(DBUS_BUS_SESSION, &err);
         if (dbus_error_is_set(&err)) {
                 dbus_error_free(&err);
-                out << "error";
+                qDebug() << "error";
         }
         if (!connection) {
-                out << "error2";
+                qDebug() << "error2";
         }
-        //qfile
-        out << "fhdaskfha" + file.fileName() << endl;
-        //result=hildon_mime_open_file(connection,(const char *) file.fileName().data());
+        qDebug() << file.fileName();
         result = 1;
-        #ifdef Q_WS_HILDON
+        #ifdef Q_WS_MAEMO_5
           result=hildon_mime_open_file(connection,"/opt/appdownloader/meta/application.install");
         #endif
-        out << result;
-        if (result != suc) {
-                out << "did not work";
+        qDebug() << result;
+        if (result != suc) {http://talk.maemo.org/showthread.php?t=50753
+                qDebug() << "did not work";
         }
     #endif
 }
 
+/**
+  receive the http-code from the login request
+  @param int code: http code
+
+  **/
+
+void MaemoApplicationHandler::loginResponse(int code) {
+    disconnect (connector,SIGNAL(getHttpResponseCode(int)),this,SLOT(loginResponse(int)));
+    if (code==302) {
+        emit loggedIn (LoggedIn);
+    }
+    else if (code==403){
+        emit loggedIn (LoggedInCredentialsError);
+    }
+    else {
+        emit loggedIn (LoggedInConnectionError);
+    }
+}
+
+/**
+  comment/rate an application:
+    - first check if logged in - error otherwise
+    - then build the url
+    - post the request
+ **/
+void MaemoApplicationHandler::commentApplication(int rating, QString comment, QString applicationname) {
+    if (isUserLoggedIn()) {
+        QByteArray postData;
+        postData.append("_qf__net_nehmer_comments=&");
+        postData.append("content="+comment+"&");
+        postData.append("rating="+ QString::number(rating) +"&");
+        postData.append("midcom_helper_datamanager2_save=Send");
+
+
+        // has to be added to the queue as well
+        if (!connector->isBusy()) {
+            connector->postRequest(QUrl ("https://maemo.org/downloads/product/Maemo5/"+applicationname),postData);
+            connect (connector,SIGNAL(getHttpResponseCode(int)),this,SLOT(commentResponse(int)));
+        }
+
+    } else {
+        emit commented (NotAuthorized);
+    }
+
+}
+
+
+/**
+  forward the response
+  **/
+void MaemoApplicationHandler::commentResponse(int resp) {
+    disconnect (connector,SIGNAL(getHttpResponseCode(int)),this,SLOT(commentResponse(int)));
+    if (resp==302)
+        emit commented(CommentSuccess);
+    else
+        emit commented(CommentConnectionError);
+}
 
 /***
   xml of application list is parsed and a list of applications given back
@@ -222,6 +420,8 @@ void MaemoApplicationHandler::parseAppListResponse(bool error,QString *content)
         QString title;
         QString rating;
         QString pre;
+        QString bigPre;
+
         QList<MaemoApplication *> *appList = new QList<MaemoApplication *>();
         while (!xmlReader.atEnd()) {
            xmlReader.readNext();
@@ -261,12 +461,16 @@ void MaemoApplicationHandler::parseAppListResponse(bool error,QString *content)
                     out << "preview-url: " + xmlReader.text().toString() << endl;
                     pre = xmlReader.text().toString();
                 }
+                else if (xmlReader.name() == "previewpic1") {
+                    xmlReader.readNext();
+                    bigPre = xmlReader.text().toString();
+                }
             } else if (xmlReader.isEndElement()) {
                 if (xmlReader.name() == "content") {
                     out << "end: " + xmlReader.name().toString()<< endl;
                     out << "************************************"<< endl;
                     out << QString::number(i)<< endl;
-                    appList->append(new MaemoApplication (name,uid,title,rating,downloads,comments,version,pre));
+                    appList->append(new MaemoApplication (name,uid,title,rating,downloads,comments,version,pre,bigPre));
                     out << QString::number(appList->length())<< endl;
                     out << "************************************"<< endl;
                     uid="";
@@ -277,12 +481,14 @@ void MaemoApplicationHandler::parseAppListResponse(bool error,QString *content)
                     title="";
                     rating="";
                     pre="";
+                    bigPre = "";
                     i++;
                 }
             }
         }
         emit applicationListReady (appList);
     }
+    delete content;
 
 }
 
@@ -295,6 +501,7 @@ void MaemoApplicationHandler::parseSingleAppResponse(bool err, QString *content)
     QTextStream out (stdout);
     out << *content <<endl;
     if (err) {
+        out << "error" << endl;
         if (mode=="testing") {
             MaemoApplicationDetails *appDetails = new MaemoApplicationDetails();
             appDetails->setName ("TESTAPP");
@@ -304,9 +511,11 @@ void MaemoApplicationHandler::parseSingleAppResponse(bool err, QString *content)
             appDetails->setDownloads ("1111");
             appDetails->setInstallFile("a");
             emit applicationDetailsReady(appDetails);
+
         } else {
             emit raiseError(content);
         }
+        out << "error2" << endl;
     } else {
         xmlReader.clear();
         xmlReader.addData (*content);
@@ -516,6 +725,44 @@ void MaemoApplicationHandler::parseCommentsOnly (QString *com) {
  }
 
 
+void MaemoApplicationHandler::parseCategoryResponse(bool err, QString *content) {
+    disconnect (connector, SIGNAL(getResponseContent(bool,QString*)),this,SLOT(parseCategoryResponse(bool,QString*)));
+    if (err) {
+        emit raiseError (content);
+    }
+    else {
+        qDebug () << "start parsing";
+        xmlReader.clear();
+        xmlReader.addData(*content);
+        QList <MaemoCategory*>* catList = new QList <MaemoCategory*>();
+        QString id;
+        QString name;
+        while (!xmlReader.atEnd()) {
+            xmlReader.readNext();
+            if (xmlReader.isStartElement()) {
+                if (xmlReader.name()=="category") {
+                    while (!(xmlReader.isEndElement()&&xmlReader.name()=="category")) {
+                        xmlReader.readNext();
+                        if (xmlReader.isStartElement()&&xmlReader.name()=="id") {
+                            xmlReader.readNext();
+                            id = xmlReader.text().toString();
+                        }
+                        else if (xmlReader.isStartElement()&&xmlReader.name()=="name") {
+                            xmlReader.readNext();
+                            name = xmlReader.text().toString();
+                        }
+                    }
+                    catList->append(new MaemoCategory(id,name));
+                }
+
+            }
+        }
+        emit categoriesReady(catList);
+    }
+    delete content;
+}
+
+
 QString MaemoApplicationHandler::queryBuilder ( QString OS,
                                                 QString cat,
                                                 int page,
@@ -571,3 +818,10 @@ QString MaemoApplicationHandler::formalizeQString (QString s) {
     s = s.replace ("\n"," ");
     return s;
 }
+
+void MaemoApplicationHandler::setHandlerMode(const QString& handlerMode)
+{
+    mode = handlerMode;
+}
+
+
